'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var jspythonInterpreter = require('jspython-interpreter');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function trimChar(text, charToRemove) {
    while (text.charAt(0) == charToRemove) {
        text = text.substring(1);
    }
    while (text.charAt(text.length - 1) == charToRemove) {
        text = text.substring(0, text.length - 1);
    }
    return text;
}

var rootFolder = process.cwd().split('\\').join('/');
var initialScope = {};
var context = {
    asserts: [],
    params: {}
};
initialScope.assert = function (condition, name, description) { return context.asserts.push({ condition: condition, name: name, description: description }); };
initialScope.showAsserts = function () { return console.table(context.asserts); };
initialScope.params = function (name) {
    var value = context.params[name];
    return value === undefined ? null : value;
};
function jsPythonForNode(options) {
    if (options === void 0) { options = {
        srcRoot: ''
    }; }
    var interpreter = jspythonInterpreter.jsPython();
    Object.assign(context.params, options.params);
    interpreter
        .registerPackagesLoader(packageLoader)
        .registerModuleLoader(moduleLoader);
    var evaluate = interpreter.evaluate;
    interpreter.evaluate = function (script, evaluationContext, entryFunctionName, moduleName) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        context.asserts.length = 0;
                        return [4 /*yield*/, initialize(options.srcRoot)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, evaluate.call(interpreter, script, evaluationContext, entryFunctionName, moduleName)];
                }
            });
        });
    };
    interpreter.evaluateFile = function (filePath, context) {
        if (context === void 0) { context = {}; }
        var script = getScript(filePath);
        return interpreter.evaluate(script, context, options.entryFunction);
    };
    return interpreter;
    function moduleLoader(filePath) {
        filePath = trimChar(trimChar(filePath, '/'), '.');
        var fileName = "".concat(options.srcRoot).concat(filePath, ".jspy");
        if (!fs__default["default"].existsSync(fileName)) {
            fileName = "".concat(options.srcRoot).concat(filePath);
        }
        if (!fs__default["default"].existsSync(fileName)) {
            fileName = "src/".concat(filePath);
        }
        try {
            var script = fs__default["default"].readFileSync(fileName, 'utf8');
            return Promise.resolve(script);
        }
        catch (e) {
            console.log('* module loader error ', (e === null || e === void 0 ? void 0 : e.message) || e);
            return Promise.reject(e);
        }
    }
    /**@type {PackageLoader} */
    function packageLoader(packageName) {
        var _a;
        try {
            if (['fs', 'path', 'readline', 'timers', 'child_process', 'util', 'zlib', 'stream', 'net', 'https', 'http', 'events', 'os', 'buffer']
                .includes(packageName)) {
                return require(packageName);
            }
            if (packageName.toLowerCase().endsWith('.js') || packageName.toLowerCase().endsWith('.json')) {
                return require("".concat(rootFolder, "/").concat(options.srcRoot).concat(packageName));
            }
            return require("".concat(rootFolder, "/node_modules/").concat(packageName));
        }
        catch (err) {
            console.log('Import Error: ', (_a = err === null || err === void 0 ? void 0 : err.message) !== null && _a !== void 0 ? _a : err);
            throw err;
        }
    }
}
function getScript(fileName) {
    if (!fs__default["default"].existsSync(fileName)) {
        throw Error("File not found");
    }
    var scripts = fs__default["default"].readFileSync(fileName, 'utf8');
    return scripts;
}
function initialize(baseSource) {
    return __awaiter(this, void 0, void 0, function () {
        var appJsPath, app;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    appJsPath = "".concat(rootFolder, "/").concat(baseSource, "app.js");
                    console.log({ rootFolder: rootFolder, baseSource: baseSource });
                    if (!fs__default["default"].existsSync(appJsPath)) {
                        appJsPath = "".concat(rootFolder, "/src/app.js");
                    }
                    if (!fs__default["default"].existsSync(appJsPath)) return [3 /*break*/, 3];
                    app = require(appJsPath);
                    if (typeof app._init == 'function') {
                        app._init();
                        delete app._init;
                    }
                    if (!(typeof app._initAsync == 'function')) return [3 /*break*/, 2];
                    return [4 /*yield*/, app._initAsync()];
                case 1:
                    _a.sent();
                    delete app._initAsync;
                    _a.label = 2;
                case 2:
                    Object.assign(initialScope, app);
                    _a.label = 3;
                case 3: return [2 /*return*/];
            }
        });
    });
}

exports.jsPythonForNode = jsPythonForNode;
